@@ -5,5 +5,5 @@ use lib 'lib', '../lib';
 use File::Stat::Moose;
 
 foreach (1..10) {
-    my $size = File::Stat::Moose->new(file=>'/etc/passwd')->size;
+    my $size = File::Stat::Moose->new( file => $0, @ARGV )->size;
 };
